"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const type_guards_1 = require("./type-guards");
function addFunctionMetadata(func, meta) {
    const funcWithMetadata = func;
    funcWithMetadata.__functionParamMetadata = meta;
    return func;
}
exports.addFunctionMetadata = addFunctionMetadata;
/**
 * Wraps a function and provides arguments from parameterProvider
 * Where possible argument values are provided when no value is passed to the new constructor.
 * This logic is determied by the parameterProvider function.
 *
 * @param type Type to be constructed
 * @param paramMetadata The types to be constructed (unfortunately this can't be done by reflection)
 * @param parameterProvider function to lookup the constructor parameter value
 */
function injectFunction(func, paramMetadata, parameterProvider) {
    addFunctionMetadata(func, paramMetadata);
    return createFunctionOptionalParams(func, parameterProvider);
}
exports.injectFunction = injectFunction;
function createFunctionOptionalParams(func, parameterProvider) {
    return wrapFunction(func, parameterProvider);
}
function wrapFunction(func, parameterProvider) {
    let params = [];
    if (type_guards_1.isDecoratedFunction(func)) {
        params = func.__functionParamMetadata;
    }
    if (params == null || params.length == null || params.length === 0) {
        return func;
    }
    return ((...args) => {
        function resolveParameter(paramReflect, index) {
            return parameterProvider(args[index], paramReflect);
        }
        const functionParams = params.map(resolveParameter);
        return func(...functionParams);
    });
}
//# sourceMappingURL=function-injector.js.map