export interface IFunctionWithMetadata<TParams extends any[]> extends FunctionWithParams<TParams> {
    __functionParamMetadata: MapMeta<TParams>;
}
/**
 * A function to provide constructor parameter values
 * If a parameter was passed to the wrapped constructor it is passed as the first argument
 * Reflect Metadata (usually the type of the parameter) is passed as the second argument
 */
export declare type ParameterProvider = (passedParameter: any, reflectData: any) => any;
export declare type Constructor = new (...args: any[]) => any;
export declare type FunctionWithParams<P extends any[] = any[]> = (...args: P) => any;
export declare type InjectedConstructor<T extends Constructor, P extends any[]> = T extends new (...args: any[]) => infer R ? new (...args: P) => R : never;
export declare type InjectedFunction<T extends FunctionWithParams, P extends any[]> = T extends (...args: any) => infer R ? (...args: P) => R : never;
export declare type Meta<T> = new (...args: any[]) => T;
export declare type MapMeta<T extends any[]> = {
    [P in keyof T]?: Meta<T[P]>;
};
export declare type MapFunctionParams<T extends any[]> = Partial<T>;
